/**
 * Copyright 2004-2005 Fabrizio Giustina.
 *
 * Licensed under the Artistic License; you may not use this file
 * except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://maven-taglib.sourceforge.net/license.html
 *
 * THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
package net.sf.maventaglib;

import java.io.File;
import java.io.FilenameFilter;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilder;

import net.sf.maventaglib.checker.Tld;
import net.sf.maventaglib.checker.TldParser;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.reporting.AbstractMavenMultiPageReport;
import org.apache.maven.reporting.MavenReport;
import org.apache.maven.reporting.MavenReportException;
import org.codehaus.doxia.site.renderer.SiteRenderer;
import org.w3c.dom.Document;

/**
 * Generates a tag reference xdoc that can be integrated in a maven generated site.
 * @goal tagreference
 * @author Fabrizio Giustina
 * @version $Id: TagreferenceMojo.java,v 1.9 2005/10/30 13:14:52 fgiust Exp $
 */
public class TagreferenceMojo
    extends AbstractMavenMultiPageReport
    implements MavenReport
{

    //    /**
    //     * Tag reference ouput file prefix.
    //     * @parameter expression="${project.build.directory}/site/tagreference"
    //     */
    //    private String tagreferenceDir;

    /**
     * tld directory path.
     * @parameter alias="taglib.src.dir" expression="${basedir}/src/main/resources/META-INF"
     */
    private String srcDir;

    /**
     * Output directory for generated docs.
     * @parameter expression="${project.reporting.outputDirectory}"
     */
    private String outputDirectory;

    /**
     * Maven project
     * @parameter expression="${project}"
     * @readonly
     */
    private MavenProject project;

    /**
     * Site renderer component.
     * @component role="org.codehaus.doxia.site.renderer.SiteRenderer"
     * @readonly
     */
    private SiteRenderer siteRenderer;

    /**
     * Setter for <code>srcDir</code>.
     * @param srcDir The srcDir to set.
     */
    public void setSrcDir( String srcDir )
    {
        this.srcDir = srcDir;
    }

    /**
     * @see org.apache.maven.reporting.AbstractMavenReport#getOutputDirectory()
     */
    protected String getOutputDirectory()
    {
        return this.outputDirectory;
    }

    /**
     * @see org.apache.maven.reporting.AbstractMavenReport#getProject()
     */
    protected MavenProject getProject()
    {
        return this.project;
    }

    /**
     * @see org.apache.maven.reporting.AbstractMavenReport#getSiteRenderer()
     */
    protected SiteRenderer getSiteRenderer()
    {
        return this.siteRenderer;
    }

    /**
     * @see org.apache.maven.reporting.MavenReport#getOutputName()
     */
    public String getOutputName()
    {
        return "tagreference"; //$NON-NLS-1$
    }

    /**
     * @see org.apache.maven.reporting.MavenReport#getName(java.util.Locale)
     */
    public String getName( Locale locale )
    {
        return Messages.getString( "Tagreference.name" ); //$NON-NLS-1$
    }

    /**
     * @see org.apache.maven.reporting.MavenReport#getDescription(java.util.Locale)
     */
    public String getDescription( Locale locale )
    {
        return Messages.getString( "Tagreference.description" ); //$NON-NLS-1$
    }

    /**
     * @see org.apache.maven.reporting.AbstractMavenReport#executeReport(java.util.Locale)
     */
    protected void executeReport( Locale locale )
        throws MavenReportException
    {

        File srcDirFile = new File( this.srcDir );
        if ( !srcDirFile.isDirectory() )
        {
            throw new MavenReportException( MessageFormat
                .format( Messages.getString( "Taglib.notadir" ), new Object[] { srcDirFile //$NON-NLS-1$
                    .getAbsolutePath() } ) );
        }

        getLog()
            .info(
                   MessageFormat
                       .format(
                                Messages.getString( "Taglib.validating" ), new Object[] { srcDirFile.getAbsolutePath() } ) ); //$NON-NLS-1$

        File[] files = srcDirFile.listFiles();

        DocumentBuilder builder;

        try
        {
            builder = XsltHelper.getDocumentBuilder();
        }
        catch ( MojoExecutionException e )
        {
            throw new MavenReportException( e.getMessage(), e );
        }

        List tldList = new ArrayList();
        for ( int i = 0; i < files.length; i++ )
        {
            File current = files[i];
            if ( !current.isDirectory() && current.getName().toLowerCase().endsWith( ".tld" ) ) //$NON-NLS-1$
            {
                Document tldDoc;
                try
                {
                    tldDoc = builder.parse( current );
                }
                catch ( Exception e )
                {
                    throw new MavenReportException( MessageFormat.format( Messages
                        .getString( "Taglib.errorwhileparsing" ), //$NON-NLS-1$
                                                                          new Object[] { current.getAbsolutePath() } ),
                                                    e );
                }

                Tld tld = TldParser.parse( tldDoc, current.getName() );
                tldList.add( tld );

                //                try
                //                {
                //                    startPage( current.getName().toLowerCase() + ".tagreference" );
                //                }
                //                catch ( Exception e )
                //                {
                //                    throw new MavenReportException( e.getMessage(), e );
                //                }
                // new TagreferenceRenderer( getSink(), locale, new Tld[] { tld }, getLog() ).render();
                //                endPage( (MultiPageSink) getSink() );

            }

            closeReport();
        }
        if ( tldList.size() == 0 )
        {
            getLog()
                .info(
                       MessageFormat
                           .format(
                                    Messages.getString( "Taglib.notldfound" ), new Object[] { srcDirFile.getAbsolutePath() } ) ); //$NON-NLS-1$
            return;
        }

        new TagreferenceRenderer( getSink(), locale, (Tld[]) tldList.toArray( new Tld[tldList.size()] ), getLog() )
            .render();

    }

    /**
     * @see org.apache.maven.reporting.AbstractMavenReport#canGenerateReport()
     */
    public boolean canGenerateReport()
    {

        File srcDirFile = new File( this.srcDir );
        if ( !srcDirFile.isDirectory() )
        {
            return false;
        }

        String[] files = srcDirFile.list( new FilenameFilter()
        {
            public boolean accept( File dir, String name )
            {
                String lcName = name.toLowerCase();
                return lcName.endsWith( "tld" ); //$NON-NLS-1$
            }
        } );

        return files.length > 0;
    }

    /**
     * @see org.apache.maven.reporting.AbstractMavenMultiPageReport#usePageLinkBar()
     */
    public boolean usePageLinkBar()
    {
        return true;
    }
}