/**
 * Copyright 2004-2005 Fabrizio Giustina.
 *
 * Licensed under the Artistic License; you may not use this file
 * except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://maven-taglib.sourceforge.net/license.html
 *
 * THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
package net.sf.maventaglib;

import java.text.MessageFormat;
import java.util.Locale;

import net.sf.maventaglib.checker.Tag;
import net.sf.maventaglib.checker.TagAttribute;
import net.sf.maventaglib.checker.TagVariable;
import net.sf.maventaglib.checker.Tld;

import org.apache.commons.lang.StringUtils;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.reporting.AbstractMavenReportRenderer;
import org.codehaus.doxia.sink.Sink;

/**
 * Validates tag handler classes fount in tlds.
 * @author Fabrizio Giustina
 * @version $Revision $ ($Author $)
 */
public class TagreferenceRenderer
    extends AbstractMavenReportRenderer
{

    private Locale locale;

    /**
     * list of Tld to check.
     */
    private Tld[] tlds;

    private Log log;

    /**
     * @param sink
     */
    public TagreferenceRenderer( Sink sink, Locale locale, Tld[] tlds, Log log )
    {
        super( sink );

        this.locale = locale;
        this.tlds = tlds;
        this.log = log;

    }

    /**
     * @see org.apache.maven.reporting.AbstractMavenReportRenderer#getTitle()
     */
    public String getTitle()
    {
        return Messages.getString( "Tagreference.title" ); //$NON-NLS-1$
    }

    /**
     * @see org.apache.maven.reporting.AbstractMavenReportRenderer#renderBody()
     */
    protected void renderBody()
    {
        sink.body();

        startSection( Messages.getString( "Tagreference.h1" ) ); //$NON-NLS-1$
        paragraph( Messages.getString( "Tagreference.intro" ) ); //$NON-NLS-1$

        sink.list();
        for ( int j = 0; j < tlds.length; j++ )
        {

            sink.listItem();
            sink.link( "#" + tlds[j].getFilename() ); //$NON-NLS-1$
            sink
                .text( MessageFormat
                    .format( Messages.getString( "Tagreference.listitem.tld" ), new Object[] { //$NON-NLS-1$
                                 StringUtils.defaultIfEmpty( tlds[j].getName(), tlds[j].getShortname() ),
                                 tlds[j].getFilename() } ) );
            sink.link_();
            sink.text( Messages.getString( "Tagreference.listitem.uri" ) + tlds[j].getUri() ); //$NON-NLS-1$
            sink.listItem_();
        }
        sink.list_();

        endSection();

        for ( int j = 0; j < tlds.length; j++ )
        {
            doTld( tlds[j] );
            sink.pageBreak();
        }

        sink.body_();
    }

    private String stripTags( String html )
    {
        if ( html == null )
        {
            return StringUtils.EMPTY;
        }
        return html.replaceAll( "\\<.*?\\>", StringUtils.EMPTY ); //$NON-NLS-1$ 
    }

    /**
     * @param tld Tld
     */
    private void doTld( Tld tld )
    {
        // new section for each tld
        sink.anchor( tld.getFilename() );
        sink.anchor_();

        startSection( StringUtils.defaultIfEmpty( tld.getName(), tld.getShortname() ) );

        this.paragraph( tld.getInfo() );
        this.paragraph( MessageFormat
            .format( Messages.getString( "Tagreference.tldversion" ), new Object[] { tld.getTlibversion() } ) ); //$NON-NLS-1$

        Tag[] tags = tld.getTags();

        if ( tags != null )
        {

            sink.list();

            for ( int j = 0; j < tags.length; j++ )
            {
                Tag tag = tags[j];

                sink.listItem();

                sink.link( "#" + tag.getName() ); //$NON-NLS-1$
                if ( tag.isDeprecated() )
                {
                    sink.italic();
                }
                sink.text( tag.getName() );

                if ( tag.isDeprecated() )
                {
                    sink.italic_();
                }
                sink.link_();

                sink.text( " " ); //$NON-NLS-1$
                sink.text( stripTags( StringUtils.substringBefore( tag.getDescription(), "." ) ) ); //$NON-NLS-1$

                sink.listItem_();
            }

            sink.list_();
        }

        sink.paragraph();
        sink.text( Messages.getString( "Tagreference.intro.required" ) ); //$NON-NLS-1$
        sink.bold();
        sink.text( Messages.getString( "Tagreference.required.marker" ) ); //$NON-NLS-1$
        sink.bold_();
        sink.paragraph_();

        if ( tags != null )
        {

            for ( int j = 0; j < tags.length; j++ )
            {
                doTag( tld.getShortname(), tags[j] );
            }
        }

        endSection();
    }

    /**
     * Checks a single tag and returns validation results.
     * @param tag Tag
     */
    private void doTag( String prefix, Tag tag )
    {

        // new subsection for each tag
        startSection( "<" + prefix + ":" + tag.getName() + ">" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

        sink.paragraph();
        sink.text( tag.getDescription() );
        sink.paragraph_();

        sink.paragraph();
        sink.text( Messages.getString( "Tagreference.cancontain" ) ); //$NON-NLS-1$
        sink.text( tag.getBodycontent() );
        sink.paragraph_();

        if ( StringUtils.isNotEmpty( tag.getExample() ) )
        {
            startSection( Messages.getString( "Tagreference.example" ) ); //$NON-NLS-1$
            sink.verbatim( true );
            sink.text( tag.getExample() );
            sink.verbatim_();
            endSection();
        }

        // variables

        // attributes
        TagAttribute[] attributes = tag.getAttributes();

        if ( attributes != null && attributes.length > 0 )
        {
            startSection( Messages.getString( "Tagreference.attributes" ) ); //$NON-NLS-1$

            startTable();
            tableHeader( new String[] {
                Messages.getString( "Tagreference.attribute.name" ), Messages.getString( "Tagreference.attribute.description" ), Messages.getString( "Tagreference.attribute.type" ) } ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

            for ( int i = 0; i < attributes.length; i++ )
            {
                TagAttribute attribute = attributes[i];

                sink.tableRow();

                sink.tableCell();
                if ( attribute.isDeprecated() )
                {
                    sink.italic();
                }
                if ( attribute.isRequired() )
                {
                    sink.bold();
                }
                sink.text( attribute.getName() );
                if ( attribute.isRequired() )
                {
                    sink.text( Messages.getString( "Tagreference.required.marker" ) ); //$NON-NLS-1$
                    sink.bold_();
                }
                if ( attribute.isDeprecated() )
                {
                    sink.italic_();
                }
                sink.tableCell_();

                sink.tableCell();
                if ( attribute.isDeprecated() )
                {
                    sink.italic();
                }
                sink.text( attribute.getDescription() );
                if ( attribute.isDeprecated() )
                {
                    sink.italic_();
                }
                sink.tableCell_();

                tableCell( StringUtils
                    .defaultIfEmpty( StringUtils.substringBefore( attribute.getType(), "java.lang." ), "String" ) ); //$NON-NLS-1$ //$NON-NLS-2$

                sink.tableRow_();

            }

            endTable();

            endSection();
        }
        else
        {
            paragraph( Messages.getString( "Tagreference.noattributes" ) ); //$NON-NLS-1$
        }

        // attributes
        TagVariable[] variables = tag.getVariables();

        if ( variables != null && variables.length > 0 )
        {
            startSection( Messages.getString( "Tagreference.variables" ) ); //$NON-NLS-1$

            startTable();
            tableHeader( new String[] {
                Messages.getString( "Tagreference.variable.name" ), Messages.getString( "Tagreference.variable.type" ), Messages.getString( "Tagreference.variable.scope" ), Messages.getString( "Tagreference.variable.description" ) } ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

            for ( int i = 0; i < variables.length; i++ )
            {
                TagVariable variable = variables[i];
                sink.tableRow();

                sink.tableCell();
                if ( variable.isDeprecated() )
                {
                    sink.italic();
                }

                if ( variable.getNameGiven() != null )
                {
                    sink.text( variable.getNameGiven() );
                    sink.text( Messages.getString( "Tagreference.variable.constant" ) ); //$NON-NLS-1$
                }
                else
                {
                    sink.text( Messages.getString( "Tagreference.variable.specifiedvia" ) ); //$NON-NLS-1$
                    sink.text( variable.getNameFromAttribute() );
                }

                if ( variable.isDeprecated() )
                {
                    sink.italic_();
                }
                sink.tableCell_();

                tableCell( StringUtils.defaultIfEmpty( StringUtils.substringBefore( variable.getType(), "java.lang." ), //$NON-NLS-1$
                                                       "String" ) ); //$NON-NLS-1$

                tableCell( variable.getScope() );

                sink.tableCell();
                if ( variable.isDeprecated() )
                {
                    sink.italic();
                }
                sink.text( variable.getDescription() );
                if ( variable.isDeprecated() )
                {
                    sink.italic_();
                }
                sink.tableCell_();

                sink.tableRow_();

            }

            endTable();

            endSection();
        }

        endSection();

    }

}