/**
 * Copyright 2004-2005 Fabrizio Giustina.
 *
 * Licensed under the Artistic License; you may not use this file
 * except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://maven-taglib.sourceforge.net/license.html
 *
 * THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
package net.sf.maventaglib.checker;

import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.lang.StringUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Reads a tld file and generates a Tld object. This "manual" parser takes in account different versions of the tlds
 * @author Fabrizio Giustina
 * @version $Revision: 1.3 $ ($Author: fgiust $)
 */
public final class TldParser
{

    /**
     * don't instantiate.
     */
    private TldParser()
    {
        // unused
    }

    /**
     * Parse a Tld object from a document.
     * @param tldDoc Document for the parsed tld
     * @param tldName of the tld file
     * @return Tld instance
     */
    public static Tld parse( Document tldDoc, String tldName )
    {

        Tld tld = new Tld();
        tld.setFilename( tldName );
        Set tags = new TreeSet();

        NodeList tagList = tldDoc.getElementsByTagName( "taglib" ).item( 0 ).getChildNodes(); //$NON-NLS-1$

        for ( int i = 0; i < tagList.getLength(); i++ )
        {
            Node tagNode = tagList.item( i );

            if ( "shortname".equals( tagNode.getNodeName() ) || "short-name".equals( tagNode.getNodeName() ) ) //$NON-NLS-1$ //$NON-NLS-2$
            {
                Node child = tagNode.getFirstChild();
                if ( child != null )
                {
                    tld.setShortname( child.getNodeValue() );
                }
            }
            else if ( "display-name".equals( tagNode.getNodeName() ) ) //$NON-NLS-1$
            {
                Node child = tagNode.getFirstChild();
                if ( child != null )
                {
                    tld.setName( child.getNodeValue() );
                }
            }
            else if ( "info".equals( tagNode.getNodeName() ) || "description".equals( tagNode.getNodeName() ) ) //$NON-NLS-1$ //$NON-NLS-2$
            {
                Node child = tagNode.getFirstChild();
                if ( child != null )
                {
                    tld.setInfo( child.getNodeValue() );
                }
            }
            else if ( "tlib-version".equals( tagNode.getNodeName() ) || "tlibversion".equals( tagNode.getNodeName() ) ) //$NON-NLS-1$ //$NON-NLS-2$
            {
                Node child = tagNode.getFirstChild();
                if ( child != null )
                {
                    tld.setTlibversion( child.getNodeValue() );
                }
            }
            else if ( "uri".equals( tagNode.getNodeName() ) ) //$NON-NLS-1$
            {
                Node child = tagNode.getFirstChild();
                if ( child != null )
                {
                    tld.setUri( child.getNodeValue() );
                }
            }
            else if ( "tag".equals( tagNode.getNodeName() ) ) //$NON-NLS-1$
            {
                Tag tag = parseTag( tagNode );
                tags.add( tag );
            }

            tld.setTags( (Tag[]) tags.toArray( new Tag[tags.size()] ) );
        }

        return tld;
    }

    /**
     * Parse a <code>tag</code> element.
     * @param tagNode Node
     * @return a Tag instance
     */
    private static Tag parseTag( Node tagNode )
    {
        Tag tag = new Tag();
        Set attributes = new TreeSet();
        Set variables = new TreeSet();
        NodeList tagAttributes = tagNode.getChildNodes();

        for ( int k = 0; k < tagAttributes.getLength(); k++ )
        {
            Node tagAttribute = tagAttributes.item( k );
            String nodeName = tagAttribute.getNodeName();

            if ( "name".equals( nodeName ) ) //$NON-NLS-1$
            {
                // tag class name
                tag.setName( tagAttribute.getFirstChild().getNodeValue() );
            }
            else if ( "description".equals( nodeName ) || "info".equals( nodeName ) ) //$NON-NLS-1$ //$NON-NLS-2$
            {
                tag.setDescription( tagAttribute.getFirstChild().getTextContent() );
            }
            else if ( "tag-class".equals( nodeName ) || "tagclass".equals( nodeName ) ) //$NON-NLS-1$ //$NON-NLS-2$
            {
                tag.setTagClass( tagAttribute.getFirstChild().getNodeValue() );
            }
            else if ( "body-content".equals( nodeName ) || "bodycontent".equals( nodeName ) ) //$NON-NLS-1$ //$NON-NLS-2$
            {
                tag.setBodycontent( tagAttribute.getFirstChild().getNodeValue() );
            }
            else if ( "example".equals( nodeName ) ) //$NON-NLS-1$
            {
                tag.setExample( tagAttribute.getTextContent() );
            }
            else if ( "tei-class".equals( nodeName ) || "teiclass".equals( nodeName ) ) //$NON-NLS-1$ //$NON-NLS-2$
            {
                // tei class name
                tag.setTeiClass( tagAttribute.getFirstChild().getNodeValue() );
            }
            else if ( "attribute".equals( nodeName ) ) //$NON-NLS-1$
            {
                TagAttribute attribute = parseTagAttribute( tagAttribute );
                attributes.add( attribute );
            }
            else if ( "variable".equals( nodeName ) ) //$NON-NLS-1$
            {
                TagVariable variable = parseTagVariable( tagAttribute );
                variables.add( variable );
            }
            tag.setAttributes( (TagAttribute[]) attributes.toArray( new TagAttribute[attributes.size()] ) );
            tag.setVariables( (TagVariable[]) variables.toArray( new TagVariable[variables.size()] ) );
        }

        tag.setDeprecated( StringUtils.contains( tag.getDescription(), "@deprecated" ) ); //$NON-NLS-1$

        return tag;
    }

    /**
     * Parse an <code>attribute</code> element.
     * @param tagAttribute Node
     * @return TagAttribute instance
     */
    private static TagAttribute parseTagAttribute( Node tagAttribute )
    {
        TagAttribute attribute = new TagAttribute();

        NodeList attributeParams = tagAttribute.getChildNodes();
        for ( int z = 0; z < attributeParams.getLength(); z++ )
        {
            Node param = attributeParams.item( z );
            if ( param.getNodeType() != Node.TEXT_NODE && param.hasChildNodes() )
            {
                if ( "name".equals( param.getNodeName() ) ) //$NON-NLS-1$
                {
                    attribute.setName( param.getFirstChild().getNodeValue() );
                }
                else if ( "type".equals( param.getNodeName() ) ) //$NON-NLS-1$
                {
                    attribute.setType( param.getFirstChild().getNodeValue() );
                }
                else if ( "description".equals( param.getNodeName() ) ) //$NON-NLS-1$
                {
                    attribute.setDescription( param.getFirstChild().getTextContent() );
                }
                else if ( "required".equals( param.getNodeName() ) ) //$NON-NLS-1$
                {
                    attribute.setRequired( StringUtils.contains( StringUtils.lowerCase( param.getTextContent() ),
                                                                 "true" ) ); //$NON-NLS-1$
                }
                else if ( "rtexprvalue".equals( param.getNodeName() ) ) //$NON-NLS-1$
                {
                    attribute.setRtexprvalue( StringUtils.contains( StringUtils.lowerCase( param.getTextContent() ),
                                                                    "true" ) ); //$NON-NLS-1$
                }
            }
        }

        attribute.setDeprecated( StringUtils.contains( attribute.getDescription(), "@deprecated" ) ); //$NON-NLS-1$

        return attribute;
    }

    /**
     * Parse an <code>attribute</code> element.
     * @param node Node
     * @return TagAttribute instance
     */
    private static TagVariable parseTagVariable( Node node )
    {
        TagVariable variable = new TagVariable();

        NodeList attributeParams = node.getChildNodes();
        for ( int z = 0; z < attributeParams.getLength(); z++ )
        {
            Node param = attributeParams.item( z );
            if ( param.getNodeType() != Node.TEXT_NODE && param.hasChildNodes() )
            {
                if ( "name-given".equals( param.getNodeName() ) ) //$NON-NLS-1$
                {
                    variable.setNameGiven( param.getFirstChild().getNodeValue() );
                }
                if ( "name-from-attribute".equals( param.getNodeName() ) ) //$NON-NLS-1$
                {
                    variable.setNameFromAttribute( param.getFirstChild().getNodeValue() );
                }
                else if ( "variable-class".equals( param.getNodeName() ) ) //$NON-NLS-1$
                {
                    variable.setType( param.getFirstChild().getNodeValue() );
                }
                else if ( "scope".equals( param.getNodeName() ) ) //$NON-NLS-1$
                {
                    variable.setScope( param.getFirstChild().getNodeValue() );
                }
                else if ( "description".equals( param.getNodeName() ) || "info".equals( param.getNodeName() ) ) //$NON-NLS-1$ //$NON-NLS-2$
                {
                    variable.setDescription( param.getFirstChild().getTextContent() );
                }

            }
        }

        variable.setDeprecated( StringUtils.contains( variable.getDescription(), "@deprecated" ) ); //$NON-NLS-1$

        return variable;
    }

}